@@ -569,8 +569,7 @@ color_balance_ok_callback (GtkWidget *widget,
 
   cbd = (ColorBalanceDialog *) data;
 
-  if (GTK_WIDGET_VISIBLE (cbd->shell))
-    gtk_widget_hide (cbd->shell);
+  gimp_dialog_hide (cbd->shell);
   
   active_tool->preserve = TRUE;
 
@@ -596,8 +595,7 @@ color_balance_cancel_callback (GtkWidget *widget,
 
   cbd = (ColorBalanceDialog *) data;
 
-  if (GTK_WIDGET_VISIBLE (cbd->shell))
-    gtk_widget_hide (cbd->shell);
+  gimp_dialog_hide (cbd->shell);
 
   if (cbd->image_map)
     {
@@ -659,7 +657,16 @@ color_balance_preview_update (GtkWidget *widget,
       color_balance_preview (cbd);
     }
   else
-    cbd->preview = FALSE;
+    {
+      cbd->preview = FALSE;
+      if (cbd->image_map)
+	{
+	  active_tool->preserve = TRUE;
+	  image_map_clear (cbd->image_map);
+	  active_tool->preserve = FALSE;
+	  gdisplays_flush ();
+	}
+    }
 }
 
 static void
