@@ -19,13 +19,21 @@
 #define __PERSPECTIVE_TOOL_H__
 
 #include "tools.h"
-#include "procedural_db.h"
+#include "transform_core.h"
 
-void *        perspective_tool_transform  (Tool *, gpointer, int);
-Tool *        tools_new_perspective_tool  (void);
-void          tools_free_perspective_tool (Tool *);
+TileManager * perspective_tool_transform   (Tool           *tool,
+					    gpointer        gdisp_ptr,
+					    TransformState  state);
+TileManager * perspective_tool_perspective (GimpImage      *gimage,
+					    GimpDrawable   *drawable,
+					    GDisplay       *gdisp,
+					    TileManager    *float_tiles,
+					    gboolean        interpolation,
+					    GimpMatrix3     matrix);
+void          perspective_find_transform   (gdouble        *coords,
+					    GimpMatrix3     matrix);
 
-/*  Procedure definition and marshalling function  */
-extern ProcRecord perspective_proc;
+Tool * tools_new_perspective_tool  (void);
+void   tools_free_perspective_tool (Tool *tool);
 
 #endif  /*  __PERSPECTIVE_TOOL_H__  */
