@@ -21,6 +21,7 @@
 #include "scroll.h"
 #include "cursorutil.h"
 #include "tools.h"
+#include "nav_window.h"
 
 
 /*  This is the delay before dithering begins
@@ -28,9 +29,6 @@
  */
 #define DITHER_DELAY 250  /*  milliseconds  */
 
-/*  Locally defined functions  */
-static int scroll_display (GDisplay *, int, int);
-
 /*  STATIC variables  */
 /*  These are the values of the initial pointer grab   */
 static int startx, starty;
@@ -66,14 +64,10 @@ void
 start_grab_and_scroll (GDisplay       *gdisp,
 		       GdkEventButton *bevent)
 {
-  GdkCursor *cursor;
-
   startx = bevent->x + gdisp->offset_x;
   starty = bevent->y + gdisp->offset_y;
 
-  cursor = gdk_cursor_new (GDK_FLEUR);
-  gdk_window_set_cursor (gdisp->canvas->window, cursor);
-  gdk_cursor_destroy (cursor);
+  change_win_cursor(gdisp->canvas->window, GDK_FLEUR);
 }
 
 
@@ -81,11 +75,7 @@ void
 end_grab_and_scroll (GDisplay       *gdisp,
 		     GdkEventButton *bevent)
 {
-  GdkCursor *cursor;
-
-  cursor = gdk_cursor_new (gdisp->current_cursor);
-  gdk_window_set_cursor (gdisp->canvas->window, cursor);
-  gdk_cursor_destroy (cursor);
+  change_win_cursor(gdisp->canvas->window, gdisp->current_cursor);
 }
 
 
@@ -93,6 +83,11 @@ void
 grab_and_scroll (GDisplay       *gdisp,
 		 GdkEventMotion *mevent)
 {
+  if(mevent && mevent->window != gdisp->canvas->window)
+    {
+      return;
+    }
+
   scroll_display (gdisp, (startx - mevent->x - gdisp->offset_x),
 		  (starty - mevent->y - gdisp->offset_y));
 }
@@ -102,7 +97,7 @@ void
 scroll_to_pointer_position (GDisplay       *gdisp,
 			    GdkEventMotion *mevent)
 {
-  int child_x, child_y;
+  double child_x, child_y;
   int off_x, off_y;
 
   off_x = off_y = 0;
@@ -119,7 +114,13 @@ scroll_to_pointer_position (GDisplay       *gdisp,
 
   if (scroll_display (gdisp, off_x, off_y))
     {
-      gdk_window_get_pointer (gdisp->canvas->window, &child_x, &child_y, NULL);
+      gdk_input_window_get_pointer (gdisp->canvas->window, mevent->deviceid,
+				    &child_x, &child_y, 
+#ifdef GTK_HAVE_SIX_VALUATORS
+                                    NULL, NULL, NULL, NULL, NULL);
+#else /* !GTK_HAVE_SIX_VALUATORS */
+				    NULL, NULL, NULL, NULL);
+#endif /* GTK_HAVE_SIX_VALUATORS */
 
       if (child_x == mevent->x && child_y == mevent->y)
 	/*  Put this event back on the queue -- so it keeps scrolling */
@@ -128,10 +129,10 @@ scroll_to_pointer_position (GDisplay       *gdisp,
 }
 
 
-static int
+int
 scroll_display (GDisplay *gdisp,
-		int       x_offset,
-		int       y_offset)
+		gint      x_offset,
+		gint      y_offset)
 {
   int old_x, old_y;
   int src_x, src_y;
@@ -202,6 +203,7 @@ scroll_display (GDisplay *gdisp,
       if (x_offset || y_offset)
 	gdisplays_flush ();
 
+      nav_window_update_window_marker(gdisp->window_nav_dialog); 
 
       /* Make sure graphics expose events are processed before scrolling
        * again */
