@@ -1,12 +1,12 @@
 /*
- * "$Id: png.c,v 1.41 2000/01/31 03:13:01 mitch Exp $"
+ * "$Id: png.c,v 1.55 2000/05/22 17:10:00 mitch Exp $"
  *
  *   Portable Network Graphics (PNG) plug-in for The GIMP -- an image
  *   manipulation program
  *
  *   Copyright 1997-1998 Michael Sweet (mike@easysw.com) and
  *   Daniel Skarda (0rfelyus@atrey.karlin.mff.cuni.cz).
- *   and 1999 Nick Lamb (njl195@zepler.org.uk)
+ *   and 1999-2000 Nick Lamb (njl195@zepler.org.uk)
  *
  *   This program is free software; you can redistribute it and/or modify
  *   it under the terms of the GNU General Public License as published by
@@ -28,6 +28,7 @@
  *   query()                     - Respond to a plug-in query...
  *   run()                       - Run the plug-in...
  *   load_image()                - Load a PNG image into a new image window.
+ *   respin_cmap()               - Re-order a Gimp colormap for PNG tRNS
  *   save_image()                - Save the specified image to a PNG file.
  *   save_ok_callback()          - Destroy the save dialog and save the image.
  *   save_compression_callback() - Update the image compression level.
@@ -54,11 +55,12 @@
 
 #include "libgimp/stdplugins-intl.h"
 
+
 /*
  * Constants...
  */
 
-#define PLUG_IN_VERSION  "1.1.11 - 29 Jan 2000"
+#define PLUG_IN_VERSION  "1.3.2 - 9 May 2000"
 #define SCALE_WIDTH      125
 
 #define DEFAULT_GAMMA    2.20
@@ -92,7 +94,10 @@ static gint	save_image                (gchar   *filename,
 					   gint32   drawable_ID,
 					   gint32   orig_image_ID);
 
-static void     init_gtk                  (void);
+static void	respin_cmap		  (png_structp pp,
+					   png_infop info,
+					   gint32   image_ID);
+
 static gint	save_dialog               (void);
 static void	save_ok_callback          (GtkWidget     *widget,
 					   gpointer       data);
@@ -137,11 +142,11 @@ query (void)
   {
     { PARAM_INT32,      "run_mode",     "Interactive, non-interactive" },
     { PARAM_STRING,     "filename",     "The name of the file to load" },
-    { PARAM_STRING,     "raw_filename", "The name of the file to load" },
+    { PARAM_STRING,     "raw_filename", "The name of the file to load" }
   };
   static GParamDef load_return_vals[] =
   {
-    { PARAM_IMAGE,      "image",        "Output image" },
+    { PARAM_IMAGE,      "image",        "Output image" }
   };
   static gint nload_args = sizeof (load_args) / sizeof (load_args[0]);
   static gint nload_return_vals = (sizeof (load_return_vals) /
@@ -160,8 +165,6 @@ query (void)
   };
   static gint nsave_args = sizeof (save_args) / sizeof (save_args[0]);
 
-  INIT_I18N ();
-
   gimp_install_procedure ("file_png_load",
 			  "Loads files in PNG file format",
 			  "This plug-in loads Portable Network Graphics (PNG) files.",
@@ -181,7 +184,7 @@ query (void)
 			   "Michael Sweet <mike@easysw.com>, Daniel Skarda <0rfelyus@atrey.karlin.mff.cuni.cz>, Nick Lamb <njl195@zepler.org.uk>",
 			   PLUG_IN_VERSION,
 			   "<Save>/PNG",
-			   "RGB*,GRAY*,INDEXED",
+			   "RGB*,GRAY*,INDEXED*",
 			   PROC_PLUG_IN,
 			   nsave_args, 0,
 			   save_args, NULL);
@@ -249,7 +252,7 @@ run (gchar   *name,
 	{
 	case RUN_INTERACTIVE:
 	case RUN_WITH_LAST_VALS:
-	  init_gtk ();
+	  gimp_ui_init ("png", FALSE);
 	  export = gimp_export_image (&image_ID, &drawable_ID, "PNG", 
 				      (CAN_HANDLE_RGB |
 				       CAN_HANDLE_GRAY |
@@ -345,10 +348,11 @@ static gint32
 load_image (gchar *filename)	/* I - File to load */
 {
   int		i,		/* Looping var */
-                trns = 0,       /* Transparency present */
-		bpp = 0,	/* Bytes per pixel */
-		image_type = 0,	/* Type of image */
-		layer_type = 0,	/* Type of drawable/layer */
+                trns,		/* Transparency present */
+		bpp,		/* Bytes per pixel */
+		image_type,	/* Type of image */
+		layer_type,	/* Type of drawable/layer */
+		empty,		/* Number of fully transparent indices */
 		num_passes,	/* Number of interlace passes in file */
 		pass,		/* Current pass in file */
 		tile_height,	/* Height of tile in GIMP */
@@ -356,8 +360,8 @@ load_image (gchar *filename)	/* I - File to load */
 		end,		/* Ending tile row */
 		num;		/* Number of rows to load */
   FILE		*fp;		/* File pointer */
-  gint32	image = -1,	/* Image */
-		layer;		/* Layer */
+  volatile gint32 image;	/* Image -- preserved against setjmp() */
+  gint32	layer;		/* Layer */
   GDrawable	*drawable;	/* Drawable for layer */
   GPixelRgn	pixel_rgn;	/* Pixel region for layer */
   png_structp	pp;		/* PNG read pointer */
@@ -369,21 +373,14 @@ load_image (gchar *filename)	/* I - File to load */
   		*alpha_ptr;	/* Temporary pointer */
 
  /*
-  * Setup the PNG data structures...
+  * PNG 0.89 and newer have a sane, forwards compatible constructor.
+  * Some SGI IRIX users will not have a new enough version though
   */
 
 #if PNG_LIBPNG_VER > 88
- /*
-  * Use the "new" calling convention...
-  */
-
   pp   = png_create_read_struct(PNG_LIBPNG_VER_STRING, NULL, NULL, NULL);
   info = png_create_info_struct(pp);
 #else
- /*
-  * SGI (and others) supply libpng-88 and not -89c...
-  */
-
   pp = (png_structp)calloc(sizeof(png_struct), 1);
   png_read_init(pp);
 
@@ -396,6 +393,10 @@ load_image (gchar *filename)	/* I - File to load */
     return image;
   }
 
+  /* initialise image here, thus avoiding compiler warnings */
+
+  image= -1;
+
  /*
   * Open the file and initialize the PNG read "engine"...
   */
@@ -425,34 +426,43 @@ load_image (gchar *filename)	/* I - File to load */
   png_read_info(pp, info);
 
  /*
-  * This is a new version with support for tRNS (alpha + colourmaps)
-  * Mail me if you have trouble -- njl195@zepler.org.uk
+  * Latest attempt, this should be my best yet :)
   */
 
-  if (info->bit_depth < 8) 
-  {
-    png_set_packing(pp);
-    if (info->color_type != PNG_COLOR_TYPE_PALETTE) {
-      png_set_expand(pp);
-    }
-  }
-  else if (info->bit_depth == 16)
+  if (info->bit_depth == 16) {
     png_set_strip_16(pp);
+  }
+
+  if (info->color_type == PNG_COLOR_TYPE_GRAY && info->bit_depth < 8) {
+    png_set_expand(pp);
+  }
+
+  if (info->color_type == PNG_COLOR_TYPE_PALETTE && info->bit_depth < 8) {
+    png_set_packing(pp);
+  }
 
  /*
-  * Turn on interlace handling...
+  * Expand G+tRNS to GA, RGB+tRNS to RGBA
   */
 
-  if (info->interlace_type)
-    num_passes = png_set_interlace_handling(pp);
-  else
-    num_passes = 1;
+  if (info->color_type != PNG_COLOR_TYPE_PALETTE &&
+                       (info->valid & PNG_INFO_tRNS)) {
+    png_set_expand(pp);
+  }
+
+ /*
+  * Turn on interlace handling... libpng returns just 1 (ie single pass)
+  * if the image is not interlaced
+  */
+
+  num_passes = png_set_interlace_handling(pp);
 
  /*
   * Special handling for INDEXED + tRNS (transparency palette)
   */
 
-  if (info->valid & PNG_INFO_tRNS &&
+#if PNG_LIBPNG_VER > 99
+  if (png_get_valid(pp, info, PNG_INFO_tRNS) &&
       info->color_type == PNG_COLOR_TYPE_PALETTE)
   {
     png_get_tRNS(pp, info, &alpha_ptr, &num, NULL);
@@ -463,7 +473,12 @@ load_image (gchar *filename)	/* I - File to load */
     for (i= num; i < 256; ++i)
       alpha[i]= 255;
     trns= 1;
+  } else {
+    trns= 0;
   }
+#else
+    trns= 0;
+#endif /* PNG_LIBPNG_VER > 99 */
 
  /*
   * Update the info structures after the transformations take effect
@@ -502,6 +517,9 @@ load_image (gchar *filename)	/* I - File to load */
         image_type = INDEXED;
         layer_type = INDEXED_IMAGE;
         break;
+    default:				/* Aie! Unknown type */
+        g_message (_("%s\nPNG unknown color model"), filename);
+        return -1;
   };
 
   image = gimp_image_new(info->width, info->height, image_type);
@@ -513,21 +531,17 @@ load_image (gchar *filename)	/* I - File to load */
 
   /*
    * Find out everything we can about the image resolution
+   * This is only practical with the new 1.0 APIs, I'm afraid
+   * due to a bug in libpng-1.0.6, see png-implement for details
    */
 
-#ifdef GIMP_HAVE_RESOLUTION_INFO
-  if (info->valid & PNG_INFO_pHYs)
-    {
-      if (info->phys_unit_type == PNG_RESOLUTION_METER)
-	gimp_image_set_resolution(image,
-				  ((double) info->x_pixels_per_unit) * 0.0254,
-				  ((double) info->y_pixels_per_unit) * 0.0254);
-      else  /*  set aspect ratio as resolution  */
-	gimp_image_set_resolution(image,
-				  ((double) info->x_pixels_per_unit) * 72.0,
-				  ((double) info->y_pixels_per_unit) * 72.0);
-    }
-#endif /* GIMP_HAVE_RESOLUTION_INFO */
+#if PNG_LIBPNG_VER > 99
+  if (png_get_valid(pp, info, PNG_INFO_pHYs)) {
+    gimp_image_set_resolution(image,
+         ((double) png_get_x_pixels_per_meter(pp, info)) * 0.0254,
+         ((double) png_get_y_pixels_per_meter(pp, info)) * 0.0254);
+  }
+#endif /* PNG_LIBPNG_VER > 99 */
 
   gimp_image_set_filename(image, filename);
 
@@ -535,8 +549,25 @@ load_image (gchar *filename)	/* I - File to load */
   * Load the colormap as necessary...
   */
 
-  if (info->color_type & PNG_COLOR_MASK_PALETTE)
+  empty= 0; /* by default assume no full transparent palette entries */
+
+  if (info->color_type & PNG_COLOR_MASK_PALETTE) {
+
+#if PNG_LIBPNG_VER > 99
+    if (png_get_valid(pp, info, PNG_INFO_tRNS)) {
+      for (empty= 0; empty < 256 && alpha[empty] == 0; ++empty);
+        /* Calculates number of fully transparent "empty" entries */
+
+      gimp_image_set_cmap(image, (guchar *) (info->palette + empty),
+                          info->num_palette - empty);
+    } else {
+      gimp_image_set_cmap(image, (guchar *)info->palette, info->num_palette);
+    }
+#else
     gimp_image_set_cmap(image, (guchar *)info->palette, info->num_palette);
+#endif /* PNG_LIBPNG_VER > 99 */
+
+  }
 
  /*
   * Create the "background" layer to hold the image...
@@ -609,7 +640,6 @@ load_image (gchar *filename)	/* I - File to load */
 
   fclose(fp);
 
-
   if (trns) {
     gimp_layer_add_alpha(layer);
     drawable = gimp_drawable_get(layer);
@@ -630,6 +660,7 @@ load_image (gchar *filename)	/* I - File to load */
 
       for (i= 0; i < tile_height * drawable->width; ++i) {
         pixel[i*2+1]= alpha [ pixel[i*2] ];
+        pixel[i*2]-= empty;
       }
 
       gimp_pixel_rgn_set_rect(&pixel_rgn, pixel, 0, begin,
@@ -659,7 +690,7 @@ save_image (gchar  *filename,	        /* I - File to save to */
 	    gint32  drawable_ID,	/* I - Current drawable */
 	    gint32  orig_image_ID)      /* I - Original image before export */
 {
-  int		i,		/* Looping var */
+  int		i, k,		/* Looping vars */
 		bpp = 0,	/* Bytes per pixel */
 		type,		/* Type of drawable/layer */
 		num_passes,	/* Number of interlace passes in file */
@@ -676,44 +707,48 @@ save_image (gchar  *filename,	        /* I - File to save to */
   gint		num_colors;	/* Number of colors in colormap */
   gint		offx, offy;	/* Drawable offsets from origin */
   guchar	**pixels,	/* Pixel rows */
+		*fixed,		/* Fixed-up pixel data */
 		*pixel;		/* Pixel data */
   gchar		*progress;	/* Title for progress display... */
   gdouble       xres, yres;	/* GIMP resolution (dpi) */
-  gdouble	gamma;
-  guchar	red, green, blue; /* For palette background */
+  gdouble	gamma;          /* GIMP gamma e.g. 2.20 */
+  png_color_16  background;     /* Background color */
+  png_time      mod_time;       /* Modification time (ie NOW) */
+  guchar	red, green,
+                blue;           /* Used for palette background */
   time_t	cutime;         /* Time since epoch */
   struct tm	*gmt;		/* GMT broken down */
 
  /*
-  * Setup the PNG data structures...
+  * PNG 0.89 and newer have a sane, forwards compatible constructor.
+  * Some SGI IRIX users will not have a new enough version though
   */
 
 #if PNG_LIBPNG_VER > 88
- /*
-  * Use the "new" calling convention...
-  */
-
-  pp   = png_create_write_struct (PNG_LIBPNG_VER_STRING, NULL, NULL, NULL);
-  info = png_create_info_struct (pp);
+  pp   = png_create_write_struct(PNG_LIBPNG_VER_STRING, NULL, NULL, NULL);
+  info = png_create_info_struct(pp);
 #else
- /*
-  * SGI (and others) supply libpng-88 and not -89c...
-  */
-
   pp = (png_structp)calloc(sizeof(png_struct), 1);
   png_write_init(pp);
 
   info = (png_infop)calloc(sizeof(png_info), 1);
-  png_info_init(info);
 #endif /* PNG_LIBPNG_VER > 88 */
 
+  if (setjmp (pp->jmpbuf))
+  {
+    g_message (_("%s\nPNG error. Couldn't save image"), filename);
+    return 0;
+  }
+
  /*
   * Open the file and initialize the PNG write "engine"...
   */
 
   fp = fopen(filename, "wb");
-  if (fp == NULL)
-    return (0);
+  if (fp == NULL) {
+    g_message (_("%s\nCouldn't create file"), filename);
+    return 0;
+  }
 
   png_init_io(pp, fp);
 
@@ -732,69 +767,21 @@ save_image (gchar  *filename,	        /* I - File to save to */
   drawable = gimp_drawable_get (drawable_ID);
   type     = gimp_drawable_type (drawable_ID);
 
-  gimp_pixel_rgn_init(&pixel_rgn, drawable, 0, 0, drawable->width,
-                      drawable->height, FALSE, FALSE);
-
  /*
-  * Set the image dimensions and save the image...
+  * Set the image dimensions, bit depth, interlacing and compression
   */
 
   png_set_compression_level (pp, pngvals.compression_level);
 
-  gamma = gimp_gamma();
-
   info->width          = drawable->width;
   info->height         = drawable->height;
   info->bit_depth      = 8;
-  info->gamma          = 1.0 / (gamma != 1.00 ? gamma : DEFAULT_GAMMA);
-  info->sig_bit.red    = 8;
-  info->sig_bit.green  = 8;
-  info->sig_bit.blue   = 8;
-  info->sig_bit.gray   = 8;
-  info->sig_bit.alpha  = 8;
   info->interlace_type = pngvals.interlaced;
-  if (!pngvals.noextras) {
-    info->valid          |= PNG_INFO_gAMA;
 
-    if (type == RGBA_IMAGE || type == GRAYA_IMAGE
-                           || type == INDEXEDA_IMAGE) {
-      gimp_palette_get_background(&red, &green, &blue);
-      info->valid |= PNG_INFO_bKGD;
-      info->background.index = 0;
-      info->background.red = red;
-      info->background.green = green;
-      info->background.blue = blue;
-      info->background.gray = (red + green + blue) / 3;
-    }
+ /*
+  * Set color type and remember bytes per pixel count 
+  */
 
-    gimp_drawable_offsets(drawable_ID, &offx, &offy);
-    if (offx != 0 || offy != 0) {
-      info->valid |= PNG_INFO_oFFs;
-      info->x_offset = offx;
-      info->y_offset = offy;
-      info->offset_unit_type= PNG_OFFSET_PIXEL;
-    }
-
-    cutime= time(NULL); /* time right NOW */
-    gmt = gmtime(&cutime);
-    info->valid |= PNG_INFO_tIME;
-    info->mod_time.year = gmt->tm_year + 1900;
-    info->mod_time.month = gmt->tm_mon + 1;
-    info->mod_time.day = gmt->tm_mday;
-    info->mod_time.hour = gmt->tm_hour;
-    info->mod_time.minute = gmt->tm_min;
-    info->mod_time.second = gmt->tm_sec;
-
-#ifdef GIMP_HAVE_RESOLUTION_INFO
-    info->valid |= PNG_INFO_pHYs;
-    gimp_image_get_resolution (orig_image_ID, &xres, &yres);
-    info->phys_unit_type = PNG_RESOLUTION_METER;
-    info->x_pixels_per_unit = (int) (xres * 39.37);
-    info->y_pixels_per_unit = (int) (yres * 39.37);
-#endif /* GIMP_HAVE_RESOLUTION_INFO */
-
-  }
- 
   switch (type)
   {
     case RGB_IMAGE :
@@ -815,20 +802,77 @@ save_image (gchar  *filename,	        /* I - File to save to */
         break;
     case INDEXED_IMAGE :
 	bpp		 = 1;
-	info->valid      |= PNG_INFO_PLTE;
         info->color_type = PNG_COLOR_TYPE_PALETTE;
-        info->palette    = (png_colorp)gimp_image_get_cmap (image_ID, &num_colors);
+	info->valid      |= PNG_INFO_PLTE;
+        info->palette= (png_colorp) gimp_image_get_cmap(image_ID, &num_colors);
         info->num_palette= num_colors;
         break;
     case INDEXEDA_IMAGE :
-	g_message (_("Can't save image with alpha\n"));
-	return 0;
+	bpp		 = 2;
+	info->color_type = PNG_COLOR_TYPE_PALETTE;
+	respin_cmap (pp, info, image_ID); /* fix up transparency */
 	break;
-
     default:
-        abort ();
+        g_message ("%s\nImage type can't be saved as PNG", filename);
+        return 0;
   };
 
+ /*
+  * Fix bit depths for (possibly) smaller colormap images
+  */
+  
+  if (info->valid & PNG_INFO_PLTE) {
+    if (info->num_palette <= 2)
+      info->bit_depth= 1;
+    else if (info->num_palette <= 4)
+      info->bit_depth= 2;
+    else if (info->num_palette <= 16)
+      info->bit_depth= 4;
+    /* otherwise the default is fine */
+  }
+
+  /* All this stuff is optional extras, if the user is aiming for smallest
+     possible file size she can turn them all off */
+
+#if PNG_LIBPNG_VER > 99
+  if (!pngvals.noextras) {
+
+    gamma = gimp_gamma();
+    png_set_gAMA(pp, info, 1.0 / (gamma != 1.00 ? gamma : DEFAULT_GAMMA));
+
+    if (type == RGBA_IMAGE || type == GRAYA_IMAGE
+                           || type == INDEXEDA_IMAGE) {
+      gimp_palette_get_background(&red, &green, &blue);
+      
+      background.index = 0;
+      background.red = red;
+      background.green = green;
+      background.blue = blue;
+      background.gray = (red + green + blue) / 3;
+      png_set_bKGD(pp, info, &background);
+    }
+
+    gimp_drawable_offsets(drawable_ID, &offx, &offy);
+    if (offx != 0 || offy != 0) {
+      png_set_oFFs(pp, info, offx, offy, PNG_OFFSET_PIXEL);
+    }
+
+    cutime= time(NULL); /* time right NOW */
+    gmt = gmtime(&cutime);
+
+    mod_time.year = gmt->tm_year + 1900;
+    mod_time.month = gmt->tm_mon + 1;
+    mod_time.day = gmt->tm_mday;
+    mod_time.hour = gmt->tm_hour;
+    mod_time.minute = gmt->tm_min;
+    mod_time.second = gmt->tm_sec;
+    png_set_tIME(pp, info, &mod_time);
+
+    gimp_image_get_resolution (orig_image_ID, &xres, &yres);
+    png_set_pHYs(pp, info, xres * 39.37, yres * 39.37, PNG_RESOLUTION_METER);
+  }
+#endif /* PNG_LIBPNG_VER > 99 */
+
   png_write_info (pp, info);
 
  /*
@@ -838,7 +882,14 @@ save_image (gchar  *filename,	        /* I - File to save to */
   if (pngvals.interlaced)
     num_passes = png_set_interlace_handling(pp);
   else
-     num_passes = 1;
+    num_passes = 1;
+
+ /*
+  * Convert unpacked pixels to packed if necessary
+  */
+
+  if (info->color_type == PNG_COLOR_TYPE_PALETTE && info->bit_depth < 8)
+    png_set_packing(pp);
 
  /*
   * Allocate memory for "tile_height" rows and save the image...
@@ -851,12 +902,12 @@ save_image (gchar  *filename,	        /* I - File to save to */
   for (i = 0; i < tile_height; i ++)
     pixels[i]= pixel + drawable->width * bpp * i;
 
+  gimp_pixel_rgn_init(&pixel_rgn, drawable, 0, 0, drawable->width,
+                      drawable->height, FALSE, FALSE);
+
   for (pass = 0; pass < num_passes; pass ++)
   {
-   /*
-    * This works if you are only writing one row at a time...
-    */
-
+      /* This works if you are only writing one row at a time... */
     for (begin = 0, end = tile_height;
          begin < drawable->height;
          begin += tile_height, end += tile_height)
@@ -867,11 +918,27 @@ save_image (gchar  *filename,	        /* I - File to save to */
 	num = end - begin;
 	
 	gimp_pixel_rgn_get_rect (&pixel_rgn, pixel, 0, begin, drawable->width, num);
+        if (info->valid & PNG_INFO_tRNS) {
+          for (i = 0; i < num; ++i) {
+	    fixed= pixels[i];
+            for (k = 0; k < drawable->width; ++k) {
+              fixed[k] = (fixed[k*2+1] > 127) ? fixed[k*2] + 1 : 0;
+            }
+          }
+       /* Forgot this case before, what if there are too many colors? */
+        } else if (info->valid & PNG_INFO_PLTE && bpp == 2) {
+          for (i = 0; i < num; ++i) {
+	    fixed= pixels[i];
+            for (k = 0; k < drawable->width; ++k) {
+              fixed[k] = fixed[k*2];
+            }
+          }
+        }
 	
 	png_write_rows (pp, pixels, num);
 	
-	gimp_progress_update (((double)pass + (double)end / (double)info->height) /
-			      (double)num_passes);
+	gimp_progress_update (((double)pass + (double)end /
+                    (double)info->height) / (double)num_passes);
       };
   };
 
@@ -902,18 +969,38 @@ save_ok_callback (GtkWidget *widget,
   gtk_widget_destroy (GTK_WIDGET (data));
 }
 
-static void 
-init_gtk (void)
-{
-  gchar **argv;
-  gint    argc;
+/* respin_cmap actually reverse fills the colormap, so that empty entries
+   are left at the FRONT, this is only needed to reduce the size of the
+   tRNS chunk when saving GIF-like transparent images with colormaps */
+
+static void respin_cmap (png_structp pp, png_infop info, gint32 image_ID) {
+  static guchar after[3 * 256];
+  gint colors;
+  guchar trans[] = { 0 };
+  guchar *before;
+
+  before= gimp_image_get_cmap(image_ID, &colors);
+
+#if PNG_LIBPNG_VER > 99
+  if (colors < 256) { /* spare space in palette :) */
+    memcpy(after + 3, before, colors * 3);
+
+    /* Apps with no natural background will use this instead, see
+       elsewhere for the bKGD chunk being written to use index 0 */
+    gimp_palette_get_background(after+0, after+1, after+2);
+
+    /* One transparent palette entry, alpha == 0 */
+    png_set_tRNS(pp, info, (png_bytep) trans, 1, NULL);
+    png_set_PLTE(pp, info, (png_colorp) after, colors + 1);
+  } else {
+    png_set_PLTE(pp, info, (png_colorp) before, colors);
+  }
+#else
+  info->valid	  |= PNG_INFO_PLTE;
+  info->palette=     (png_colorp) before;
+  info->num_palette= colors;
+#endif /* PNG_LIBPNG_VER > 99 */
 
-  argc    = 1;
-  argv    = g_new (gchar *, 1);
-  argv[0] = g_strdup ("png");
-  
-  gtk_init (&argc, &argv);
-  gtk_rc_parse (gimp_gtkrc ());
 }
 
 static gint
@@ -927,7 +1014,7 @@ save_dialog (void)
   GtkObject *scale_data;
 
   dlg = gimp_dialog_new (_("Save as PNG"), "png",
-			 gimp_plugin_help_func, "filters/png.html",
+			 gimp_standard_help_func, "filters/png.html",
 			 GTK_WIN_POS_MOUSE,
 			 FALSE, TRUE, FALSE,
 
