@@ -1,5 +1,5 @@
 /*
- * "$Id: print.h,v 1.12 2000/01/26 16:00:48 neo Exp $"
+ * "$Id: print.h,v 1.13 2000/04/01 18:03:17 mitch Exp $"
  *
  *   Print plug-in header file for the GIMP.
  *
@@ -236,6 +236,13 @@ extern void	indexed_to_gray(unsigned char *, unsigned char *, int, int,
 				lut_t *, unsigned char *, float);
 #endif
 
+int                   known_printers(void);
+const printer_t      *get_printers(void);
+const printer_t      *get_printer_by_index(int);
+const printer_t      *get_printer_by_long_name(const char *);
+const printer_t      *get_printer_by_driver(const char *);
+int                   get_printer_index_by_driver(const char *);
+
 /*
- * End of "$Id: print.h,v 1.12 2000/01/26 16:00:48 neo Exp $".
+ * End of "$Id: print.h,v 1.13 2000/04/01 18:03:17 mitch Exp $".
  */
