@@ -36,11 +36,12 @@
 #include "libgimp/gimp.h"
 #include "libgimp/gimpui.h"
 
-#include "libgimp/stdplugins-intl.h"
+#include "script-fu-intl.h"
 
 #include "siod.h"
 #include "script-fu-console.h"
-#include <plug-ins/dbbrowser/dbbrowser.h>
+
+#include <plug-ins/dbbrowser/dbbrowser_utils.h>
 
 #ifdef G_OS_WIN32
 #include <fcntl.h>
@@ -177,35 +178,29 @@ script_fu_console_interface (void)
   GtkWidget *vsb;
   GtkWidget *table;
   GtkWidget *hbox;
-  gchar **argv;
-  gint argc;
-
-  argc = 1;
-  argv = g_new (gchar *, 1);
-  argv[0] = g_strdup ("script-fu");
-
-  gtk_init (&argc, &argv);
-  gtk_rc_parse (gimp_gtkrc ());
 
   INIT_I18N_UI();
 
+  gimp_ui_init ("script-fu", FALSE);
+
   dlg = gtk_dialog_new ();
   gtk_window_set_title (GTK_WINDOW (dlg), _("Script-Fu Console"));
+  gimp_help_connect_help_accel (dlg, gimp_standard_help_func, 
+				"filters/script-fu.html");
   gtk_signal_connect (GTK_OBJECT (dlg), "destroy",
 		      (GtkSignalFunc) script_fu_close_callback,
 		      NULL);
-  gtk_signal_connect (GTK_OBJECT (dlg),
-		      "destroy",
+  gtk_signal_connect (GTK_OBJECT (dlg), "destroy",
 		      GTK_SIGNAL_FUNC (gtk_widget_destroyed),
 		      &dlg);
   gtk_container_border_width (GTK_CONTAINER (GTK_DIALOG (dlg)->vbox), 2);
-  gtk_container_border_width (GTK_CONTAINER (GTK_DIALOG (dlg)->action_area), 2);
+  gtk_container_border_width (GTK_CONTAINER (GTK_DIALOG (dlg)->action_area), 0);
 
   /*  Action area  */
   button = gtk_button_new_with_label (_("Close"));
-  gtk_signal_connect (GTK_OBJECT (button), "clicked",
-                      (GtkSignalFunc) script_fu_close_callback,
-                      NULL);
+  gtk_signal_connect_object (GTK_OBJECT (button), "clicked",
+			     (GtkSignalFunc) gtk_widget_destroy,
+			     (gpointer)dlg);
   gtk_box_pack_start (GTK_BOX (GTK_DIALOG (dlg)->action_area), button, TRUE, TRUE, 0);
   gtk_widget_show (button);
 
@@ -246,7 +241,9 @@ script_fu_console_interface (void)
   gtk_text_insert (GTK_TEXT (cint.console), cint.font_strong, NULL, NULL,
 		   "The GIMP - GNU Image Manipulation Program\n\n", -1);
   gtk_text_insert (GTK_TEXT (cint.console), cint.font_emphasis, NULL, NULL,
-		   "Copyright (C) 1995 Spencer Kimball and Peter Mattis\n", -1);
+		   "Copyright (C) 1995-2000\n", -1);
+  gtk_text_insert (GTK_TEXT (cint.console), cint.font_emphasis, NULL, NULL,
+		   "Spencer Kimball, Peter Mattis and the GIMP Development Team\n", -1);
   gtk_text_insert (GTK_TEXT (cint.console), cint.font_weak, NULL, NULL,
 		   "\n", -1);
   gtk_text_insert (GTK_TEXT (cint.console), cint.font_weak, NULL, NULL,
@@ -301,7 +298,7 @@ script_fu_console_interface (void)
   
   gtk_box_pack_start (GTK_BOX (hbox), cint.cc, 
 		      TRUE, TRUE, 0);
-  gtk_widget_set_usize (cint.cc, (ENTRY_WIDTH*5)/6, 0); 
+/*    gtk_widget_set_usize (cint.cc, (ENTRY_WIDTH*5)/6, 0);  */
   GTK_WIDGET_SET_FLAGS (cint.cc, GTK_CAN_DEFAULT);
   gtk_widget_grab_default (cint.cc);
   gtk_signal_connect (GTK_OBJECT (cint.cc), "key_press_event",
@@ -309,7 +306,7 @@ script_fu_console_interface (void)
 		      NULL);
 
   button = gtk_button_new_with_label (_("Browse..."));
-  gtk_widget_set_usize (button, (ENTRY_WIDTH)/6, 0);
+/*    gtk_widget_set_usize (button, (ENTRY_WIDTH)/6, 0); */
   gtk_box_pack_start (GTK_BOX (hbox), button, 
 		      FALSE, TRUE, 0);
   gtk_signal_connect (GTK_OBJECT (button), "clicked",
